@@ -1,4 +1,4 @@
-/* Copyright 2018 The TensorFlow Authors. All Rights Reserved.
+/* Copyright 2018 The OpenXLA Authors.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -13,24 +13,30 @@ See the License for the specific language governing permissions and
 limitations under the License.
 ==============================================================================*/
 
-#ifndef TENSORFLOW_COMPILER_XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
-#define TENSORFLOW_COMPILER_XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
+#ifndef XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
+#define XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
 
+#include <cstdint>
 #include <functional>
+#include <memory>
 #include <optional>
+#include <string>
 #include <utility>
 #include <vector>
 
 #include "absl/algorithm/container.h"
 #include "absl/container/inlined_vector.h"
+#include "absl/log/check.h"
+#include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
-#include "tensorflow/compiler/xla/pjrt/compile_options.pb.h"
-#include "tensorflow/compiler/xla/service/compilation_environments.h"
-#include "tensorflow/compiler/xla/service/computation_placer.h"
-#include "tensorflow/compiler/xla/shape.h"
-#include "tensorflow/compiler/xla/xla.pb.h"
-#include "tensorflow/compiler/xla/xla_data.pb.h"
-#include "tensorflow/tsl/platform/threadpool.h"
+#include "absl/types/span.h"
+#include "xla/pjrt/compile_options.pb.h"
+#include "xla/pjrt/distributed/key_value_store_interface.h"
+#include "xla/service/compilation_environments.h"
+#include "xla/service/computation_placer.h"
+#include "xla/shape.h"
+#include "xla/tsl/platform/threadpool.h"
+#include "xla/xla.pb.h"
 
 namespace stream_executor {
 
@@ -87,10 +93,6 @@ class ExecutableBuildOptions {
       se::DeviceMemoryAllocator* allocator);
   se::DeviceMemoryAllocator* device_allocator() const;
 
-  // Returns a string representation of the build options, suitable for
-  // debugging.
-  std::string ToString() const;
-
   // The number of replicas of this computation that are to be executed.
   // Defaults to 1.
   int num_replicas() const { return num_replicas_; }
@@ -124,6 +126,22 @@ class ExecutableBuildOptions {
   ExecutableBuildOptions& set_auto_spmd_partitioning_mesh_ids(
       std::vector<int64_t> mesh_ids);
 
+  float exec_time_optimization_effort() const {
+    return exec_time_optimization_effort_;
+  }
+  ExecutableBuildOptions& set_exec_time_optimization_effort(
+      float exec_time_optimization_effort) {
+    exec_time_optimization_effort_ = exec_time_optimization_effort;
+    return *this;
+  }
+
+  float memory_fitting_effort() const { return memory_fitting_effort_; }
+  ExecutableBuildOptions& set_memory_fitting_effort(
+      float memory_fitting_effort) {
+    memory_fitting_effort_ = memory_fitting_effort;
+    return *this;
+  }
+
   bool deduplicate_hlo() const { return deduplicate_hlo_; }
   ExecutableBuildOptions& set_deduplicate_hlo(bool deduplicate_hlo);
 
@@ -138,6 +156,7 @@ class ExecutableBuildOptions {
     CHECK(device_assignment_.has_value());
     return device_assignment_.value();
   }
+  void clear_device_assignment() { device_assignment_.reset(); }
 
   // Whether input and output buffers are aliased if the associated parameter is
   // passed-through XLA modules without being changed.
@@ -157,12 +176,33 @@ class ExecutableBuildOptions {
     return *this;
   }
 
+  absl::Span<const bool> allow_spmd_sharding_propagation_to_parameters() const {
+    return allow_spmd_sharding_propagation_to_parameters_;
+  }
   absl::Span<const bool> allow_spmd_sharding_propagation_to_output() const {
     return allow_spmd_sharding_propagation_to_output_;
   }
+  bool any_allow_spmd_sharding_propagation_to_parameters() const {
+    return absl::c_linear_search(allow_spmd_sharding_propagation_to_parameters_,
+                                 true);
+  }
   bool any_allow_spmd_sharding_propagation_to_output() const {
-    return absl::c_any_of(allow_spmd_sharding_propagation_to_output_,
-                          [](bool b) { return b; });
+    return absl::c_linear_search(allow_spmd_sharding_propagation_to_output_,
+                                 true);
+  }
+  // Allows sharding propagation to propagate to the inputs. This changes the
+  // input shape of the computation (which is undesirable), but it can be used
+  // to allow to run partial compilation to determine what would be the input
+  // sharding of a computation if XLA would be allowed to propagate the sharding
+  // which can be used by higher level framework as a way to query intermediate
+  // sharding of operations when multiple computation would be chained and
+  // merged together.
+  ExecutableBuildOptions& set_allow_spmd_sharding_propagation_to_parameters(
+      absl::Span<const bool> allow_spmd_sharding_propagation_to_parameters) {
+    allow_spmd_sharding_propagation_to_parameters_.assign(
+        allow_spmd_sharding_propagation_to_parameters.begin(),
+        allow_spmd_sharding_propagation_to_parameters.end());
+    return *this;
   }
   // Allows sharding propagation to propagate to the outputs. This changes the
   // output shape of the computation (which is undesirable), but it can be used
@@ -189,10 +229,8 @@ class ExecutableBuildOptions {
     return *this;
   }
 
-  StatusOr<ExecutableBuildOptionsProto> ToProto() const;
-
   using LayoutCanonicalizationCallback =
-      std::function<StatusOr<std::pair<std::vector<Shape>, Shape>>(
+      std::function<absl::StatusOr<std::pair<std::vector<Shape>, Shape>>(
           const HloModule& module)>;
   void set_layout_canonicalization_callback(
       LayoutCanonicalizationCallback callback) {
@@ -202,6 +240,48 @@ class ExecutableBuildOptions {
     return layout_canonicalization_callback_;
   }
 
+  absl::string_view fdo_profile() const { return fdo_profile_; }
+  void set_fdo_profile(std::string fdo_profile) {
+    fdo_profile_ = std::move(fdo_profile);
+  }
+  std::string* mutable_fdo_profile() { return &fdo_profile_; }
+
+  // The amount of device memory available for the executable.
+  int64_t device_memory_size() const { return device_memory_size_; }
+  ExecutableBuildOptions& set_device_memory_size(int64_t device_memory_size) {
+    device_memory_size_ = device_memory_size;
+    return *this;
+  }
+
+  bool use_shardy_partitioner() const { return use_shardy_partitioner_; }
+  ExecutableBuildOptions& set_use_shardy_partitioner(
+      bool use_shardy_partitioner) {
+    use_shardy_partitioner_ = use_shardy_partitioner;
+    return *this;
+  }
+
+  // Returns a string representation of the build options, suitable for
+  // debugging.
+  std::string ToString() const;
+
+  absl::StatusOr<ExecutableBuildOptionsProto> ToProto() const;
+
+  int process_index() const { return process_index_; }
+  void set_process_index(const int process_index) {
+    process_index_ = process_index;
+  }
+  int process_count() const { return process_count_; }
+  void set_process_count(const int process_count) {
+    process_count_ = process_count;
+  }
+
+  std::shared_ptr<KeyValueStoreInterface> key_value_store() const {
+    return key_value_store_;
+  }
+  void set_key_value_store(std::shared_ptr<KeyValueStoreInterface> kv_store) {
+    key_value_store_ = kv_store;
+  }
+
  private:
   int device_ordinal_ = -1;
   Shape result_layout_;
@@ -215,18 +295,28 @@ class ExecutableBuildOptions {
   bool use_auto_spmd_partitioning_ = false;
   std::vector<int64_t> auto_spmd_partitioning_mesh_shape_;
   std::vector<int64_t> auto_spmd_partitioning_mesh_ids_;
+  float exec_time_optimization_effort_ = 0.0f;
+  float memory_fitting_effort_ = 0.0f;
   bool deduplicate_hlo_ = false;
   bool broadcast_replicated_params_ = false;
   std::optional<DeviceAssignment> device_assignment_;
   bool alias_passthrough_params_ = false;
   bool run_backend_only_ = false;
+  absl::InlinedVector<bool, 1> allow_spmd_sharding_propagation_to_parameters_ =
+      {false};
   absl::InlinedVector<bool, 1> allow_spmd_sharding_propagation_to_output_ = {
       false};
   tsl::thread::ThreadPool* compile_thread_pool_ = nullptr;
   LayoutCanonicalizationCallback layout_canonicalization_callback_;
+  std::string fdo_profile_;
+  int64_t device_memory_size_ = 0;
+  bool use_shardy_partitioner_ = false;
+  int process_index_ = 0;
+  int process_count_ = 1;
+  std::shared_ptr<KeyValueStoreInterface> key_value_store_;
 };
 
-StatusOr<ExecutableBuildOptions> ExecutableBuildOptionsFromProto(
+absl::StatusOr<ExecutableBuildOptions> ExecutableBuildOptionsFromProto(
     const ExecutableBuildOptionsProto& input);
 
 // Creates an ExecutionOptions based on a given ExecutableBuildOptions and
@@ -237,4 +327,4 @@ ExecutionOptions CreateExecutionOptions(
 
 }  // namespace xla
 
-#endif  // TENSORFLOW_COMPILER_XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
+#endif  // XLA_CLIENT_EXECUTABLE_BUILD_OPTIONS_H_
