@@ -15,33 +15,21 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_PROFILER_LIB_PROFILER_INTERFACE_H_
 #define TENSORFLOW_CORE_PROFILER_LIB_PROFILER_INTERFACE_H_
 
+#include "absl/base/macros.h"
 #include "tensorflow/core/platform/status.h"
 #include "tensorflow/core/profiler/protobuf/xplane.pb.h"
+#include "tsl/profiler/lib/profiler_interface.h"
+
+// TODO: b/323943471 - This macro should eventually be provided by Abseil.
+#ifndef ABSL_DEPRECATE_AND_INLINE
+#define ABSL_DEPRECATE_AND_INLINE()
+#endif
 
 namespace tensorflow {
 namespace profiler {
 
-// Interface for tensorflow profiler plugins.
-//
-// ProfileSession calls each of these methods at most once per instance, and
-// implementations can rely on that guarantee for simplicity.
-//
-// Thread-safety: Implementations are only required to be go/thread-compatible.
-// ProfileSession is go/thread-safe and synchronizes access to ProfilerInterface
-// instances.
-class ProfilerInterface {
- public:
-  virtual ~ProfilerInterface() = default;
-
-  // Starts profiling.
-  virtual Status Start() = 0;
-
-  // Stops profiling.
-  virtual Status Stop() = 0;
-
-  // Saves collected profile data into XSpace.
-  virtual Status CollectData(XSpace* space) = 0;
-};
+using ProfilerInterface ABSL_DEPRECATE_AND_INLINE() =
+    tsl::profiler::ProfilerInterface;  // NOLINT
 
 }  // namespace profiler
 }  // namespace tensorflow
